# -.- coding: utf-8 -.-

# Zeitgeist Indicator
#
# Copyright © 2010 Seif Lotfy <seif@lotfy.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Create a session bus.
import dbus
import gtk
import gobject
import wnck
import time
import appindicator
from zeitgeist.client import ZeitgeistClient
from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, \
	 ResultType, TimeRange, StorageState
import mimematcher

CLIENT = ZeitgeistClient()

TEMPO = 30

bus = dbus.SessionBus()
# Create an object that will proxy for a particular remote object.
OBJECT = bus.get_object("org.wncksync.Matcher", # Connection name
							"/org/wncksync/Matcher")
IFACE = dbus.Interface(OBJECT, "org.wncksync.Matcher")

class Relator(gobject.GObject):
	def __init__(self):
		gobject.GObject.__init__(self)
		self.screen = wnck.screen_get_default()
		self.apps = {}
		self.screen.force_update()
		self._set_up_indicator()
		self._update()
		self.screen.connect("active_window_changed", self._active_window_changed)
		
	def _set_up_indicator(self):
		self.ind = appindicator.Indicator ("zeitgeist-indicator",
								"stock_open",
								appindicator.CATEGORY_APPLICATION_STATUS)
		self.ind.set_status (appindicator.STATUS_ACTIVE)
	
	def _update_indicator(self, app):
		interps = self._get_mimetypes_for_app(app)
		events = []
		for interp in interps:
			event = Event()
			subj = Subject()
			subj.set_interpretation(interp)
			event.set_subjects([subj])
			events.append(event)
		
		def callback(events):
			menu = gtk.Menu()
			for event in events:
				subject = event.subjects[0]
				menu_items = gtk.MenuItem(subject.text)
				menu.append(menu_items)
				# this is where you would connect your menu item up with a function:
				# menu_items.connect("activate", self.menuitem_response, buf)
				# show the items
				menu_items.show()
			self.ind.set_menu(menu)
			
		CLIENT.find_events_for_templates(events, callback,
											[0, time.time()*1000], num_events=7,
											result_type=2)
		

	def _update(self):
		active_win = self.screen.get_active_window()
		xid = active_win.get_xid()
		app = IFACE.DesktopFileForXid(xid)
		self._update_indicator(app)
		
	def _get_mimetypes_for_app(self, app):
		if self.apps.has_key(app):
			return self.apps[app]
		f = file(app)
		interps = []
		mimetypes = []
		for line in f:
			line = line.strip()
			if line.startswith("MimeType"):
				mimetypes = line.split("=")
				for m in mimetypes[1].split(";"):
					print "***", m
					interp = mimematcher.get_interpretation_for_mimetype(m)
					if not interp in interps:
						interps.append(interp)
				break
		f.close()
		self.apps[app] = interps
		return self.apps[app]
		
	def _active_window_changed(self, screen, window):
		gobject.timeout_add(TEMPO, self._update)

